@@ -84,8 +84,8 @@ sub profile_default
 		buttonProfile  => {},
 		hScrollBarProfile => {},
 		vScrollBarProfile => {},
-		listDelegations   => [qw(Leave SelectItem MouseUp Click KeyDown)],
-		editDelegations   => [qw(FontChanged Create Setup KeyDown KeyUp Change Leave MouseWheel)],
+		listDelegations   => [qw(Leave SelectItem MouseUp Click KeyDown Paint)],
+		editDelegations   => [qw(FontChanged Create Setup KeyDown KeyUp Change Leave MouseWheel MouseDown)],
 		buttonDelegations => [qw(ColorChanged FontChanged MouseDown MouseClick
 			MouseUp MouseMove MouseEnter MouseLeave Paint Enable Disable)],
 	}
@@ -192,8 +192,11 @@ sub init
 sub check_auto_size
 {
 	my $self = $_[0];
-	$self-> geomHeight( $self-> {edit}-> default_geom_height )
-		if $self-> {autoHeight} && $self->{style} != cs::Simple && $self->{edit}->can('default_geom_height');
+	if ($self-> {autoHeight} && $self->{style} != cs::Simple && $self->{edit}->can('default_geom_height')) {
+		$self-> {list}-> growMode(0);
+		$self-> geomHeight( $self-> {edit}-> default_geom_height );
+		$self-> {list}-> growMode(gm::Client);
+	}
 }
 
 sub on_create
@@ -250,6 +253,25 @@ sub List_KeyDown
 	}
 }
 
+sub List_Paint
+{
+	my ( $self, $list, $canvas) = @_;
+	if ( $self->skin eq 'flat' && $list->isa('Prima::Widget::GroupScroller')) {
+		my $bw = $list->borderWidth;
+		if ( $bw > 0 ) {
+			my @sz = $list->size;
+			my $cc = $list->color;
+			$canvas-> color($list->dark3DColor);
+			$canvas-> bar(0,$sz[1]-$bw-1,$bw-1,$sz[1]-1);
+			$canvas-> bar($sz[0]-$bw-1,$sz[1]-$bw-1,$sz[0]-1,$sz[1]-1);
+			$canvas-> color($cc);
+			$canvas-> clear($bw, $sz[1]-$bw-1, $sz[0]-$bw-1, $sz[1]-1);
+			$canvas-> clipRect(0,0,$sz[0]-1,$sz[1]-$bw-1);
+		}
+	}
+	$list->on_paint($canvas);
+}
+
 sub Button_ColorChanged
 {
 	my $self = shift;
@@ -346,25 +368,42 @@ sub Button_Paint
 	my @clr    = $ena ?
 		( $self-> color, $self-> backColor) :
 		( $self-> disabledColor, $self-> disabledBackColor);
-	$clr[1] = $self->prelight_color($clr[1]) if $self->{prelight};
+	my $rc;
 	my $lv = $owner-> listVisible;
-	my ( $rc, $lc) = ( $self-> light3DColor, $self-> dark3DColor);
-	( $rc, $lc) = ( $lc, $rc) if $lv;
-	$self-> rect_bevel(
-		$canvas, 0, 0, $w-1, $h-1,
-		fill => $self-> new_gradient(
-			palette  => [ $self-> dark3DColor, $clr[1], $self-> light3DColor ],
-			spline   => [0,0.5,1,0.5],
-			vertical => 0,
-		),
-		width => 2
-	);
 	my @triangle = fix_triangle( 4, $h * 0.6, $w/2, $h * 0.4, $w - 4, $h * 0.6 );
-	if ( $ena) {
-		$canvas-> color( $rc);
-		$canvas-> translate(1,-1);
-		$canvas-> fillpoly(\@triangle);
-		$canvas-> translate(0,0);
+	if ( $self-> skin eq 'flat') {
+		my $hbc = $self-> hiliteBackColor;
+		$canvas-> graphic_context( sub {
+			$canvas-> color( $self-> dark3DColor ); 
+			$canvas-> rectangle(-1, 0, $w-1, $h-1); # yes, -1
+			my $c = cl::blend(
+				$self->map_color( $hbc ),
+				$self->map_color( $clr[1] ),
+				$self->{prelight} ? 0.33 : 0.66
+			);
+			$canvas-> color( $c );
+			$canvas-> bar( 1 - 1, 1, $w-2, $h-2);
+		});
+		$clr[0] = $self->hiliteColor;
+		$canvas-> translate(1,-1) if $lv;
+	} else {
+		$clr[1] = $self->prelight_color($clr[1]) if $self->{prelight};
+		$self-> rect_bevel(
+			$canvas, 0, 0, $w-1, $h-1,
+			fill => $self-> new_gradient(
+				palette  => [ $self-> dark3DColor, $clr[1], $self-> light3DColor ],
+				spline   => [0,0.5,1,0.5],
+				vertical => 0,
+			),
+			width => 2
+		);
+		$rc = $lv ? $self-> dark3DColor : $self-> light3DColor;
+		if ( $ena) {
+			$canvas-> color( $rc);
+			$canvas-> translate(1,-1);
+			$canvas-> fillpoly(\@triangle);
+			$canvas-> translate(0,0);
+		}
 	}
 	$canvas-> color( $clr[0]);
 	$canvas-> fillpoly(\@triangle);
@@ -406,7 +445,9 @@ sub InputLine_KeyDown
 		( $key != kb::Space) &&
 		( $key != kb::Backspace)
 	) {
-		return if $key == kb::Tab || $key == kb::BackTab || $key == kb::NoKey;
+		return if $key == kb::Tab || $key == kb::BackTab || $key == kb::NoKey ||
+			(($key & kb::ShiftL & kb::ShiftR) != 0);
+		return unless $code;
 		$edit-> {incline} = '';
 		$self-> listVisible(1), $edit-> clear_event
 			if $key == kb::Down && $_[0]-> {style} != cs::Simple;
@@ -534,6 +575,13 @@ sub InputLine_Change
 	$list-> {interaction} = undef;
 }
 
+sub InputLine_MouseDown
+{
+	my ( $self, $edit, $mod, $x, $y) = @_;
+	# there's no literal input in cs::DropDown, so don't confuse the user
+	$self-> listVisible(0) if $self->style == cs::DropDown && $self->listVisible;
+}
+
 sub InputLine_MouseWheel
 {
 	my ( $self, $edit, $mod, $x, $y, $z) = @_;
