@@ -103,7 +103,12 @@ sub matrix
 {
 	my ( $self, @m ) = @_;
 	if ( (1 == @m) && ref($m[0]) ) {
-		$self->cmd( matrix => @{$m[0]} );
+		my $matrix = $m[0];
+		if ( UNIVERSAL::isa($matrix, 'Prima::Matrix')) {
+			$self->cmd( matrix => @{$matrix->get} );
+		} else {
+			$self->cmd( matrix => @$matrix );
+		}
 	} elsif ( @m == 6 ) {
 		$self->cmd( matrix => @m );
 	} else {
@@ -244,6 +249,7 @@ sub arc
 	@_ > 5 or Carp::croak('bad parameters to arc');
 	my ( $cx, $cy, $dx, $dy, $from, $to, $tilt) = @_;
 	return $self if $from == $to;
+	$dx--, $dy-- unless $self->{subpixel};
 	if ( $tilt // 0.0 ) {
 		return $self-> save->
 			scale( $dx / 2, $dy / 2)->
@@ -265,6 +271,7 @@ sub rarc
 	@_ > 3 or Carp::croak('bad parameters to arcto');
 	my ( $dx, $dy, $from, $to, $tilt) = @_;
 	return $self if $from == $to;
+	$dx--, $dy-- unless $self->{subpixel};
 	$self->save;
 	$self->scale( $dx / 2, $dy / 2);
 	$self->rotate( $tilt // 0.0);
@@ -278,6 +285,7 @@ sub ellipse
 	@_ > 2 or Carp::croak('bad parameters to ellipse');
 	my ( $cx, $cy, $dx, $dy, $tilt) = @_;
 	$dy //= $dx;
+	$dx--, $dy-- unless $self->{subpixel};
 	$self-> save->
 		matrix( $dx / 2, 0, 0, $dy / 2, $cx, $cy )->
 		rotate( $tilt // 0.0)->
@@ -290,6 +298,7 @@ sub chord
 	my $self = shift;
 	@_ == 6 or Carp::croak('bad parameters to chord');
 	my ( $cx, $cy, $dx, $dy, $start, $end) = @_;
+	$dx--, $dy-- unless $self->{subpixel};
 	$self-> save->
 		matrix( $dx / 2, 0, 0, $dy / 2, $cx, $cy )->
 		circular_arc( $start, $end )->
@@ -321,6 +330,7 @@ sub sector
 	my $self = shift;
 	@_ == 6 or Carp::croak('bad parameters to sector');
 	my ( $cx, $cy, $dx, $dy, $start, $end) = @_;
+	$dx--, $dy-- unless $self->{subpixel};
 	$self-> save->
 		matrix( $dx / 2, 0, 0, $dy / 2, $cx, $cy )->
 		line(0,0)->
@@ -338,7 +348,7 @@ sub round_rect
 	$dx = $maxd if $dx > $maxd;
 	$dy = $maxd if $dy > $maxd;
 	my $d = ( $dx < $dy ) ? $dx : $dy;
-	my $r = $self->{subpixel} ? $d/2 : int($d/2);
+	my $r = $self->{subpixel} ? $d/2 : ((int($d/4) * 2) + 1);
 #  plots roundrect:
 # A'        B'
 #  /------\
@@ -599,7 +609,10 @@ sub _arc
 	$xopt{precision} = $self->{curr}->{precision} // 24;
 	$xopt{precision} = $scaling if $xopt{precision} // 24 > $scaling;
 	$xopt{integer}   = !$self->{subpixel};
-	$xopt{precision} *= 2 if $self->{subpixel} && $self->canvas->get_paint_state == ps::Disabled; # emulated AA for images
+	$xopt{precision} *= 2 if
+		$self->{subpixel} &&
+		$self->canvas && 
+		$self->canvas->get_paint_state == ps::Disabled; # emulated AA for images
 	for my $set ( @$nurbset ) {
 		my ( $points, @options ) = @$set;
 		my $poly = $self-> matrix_apply( $points );
@@ -1070,6 +1083,10 @@ sub widen_old
 
 	my @dst;
 	my $lw2 = $lw / 2;
+
+	# classic widening algorithm doesn't do that (I believe) because there's no fm::Overlay - but Prima has it
+	$lw2 -= 0.5 unless $self->{subpixel};
+
 	for ( @$pp ) {
 		next unless @$_;
 
