@@ -27,7 +27,7 @@ sub profile_default
 		autoHeight     => 1,
 		autoSelect     => 1,
 		autoTab        => 0,
-		borderWidth    => 2,
+		borderWidth    => undef,
 		charOffset     => 0,
 		cursorVisible  => 1,
 		cursorSize     => [ Prima::Application-> get_default_cursor_width, $font-> { height}],
@@ -72,6 +72,10 @@ sub profile_check_in
 		if exists $p-> {height} || exists $p-> {size} || exists $p-> {rect} || ( exists $p-> {top} && exists $p-> {bottom});
 	$p-> {alignment} = ( $p->{textDirection} // $default->{textDirection} ) ?
 		ta::Right : ta::Left unless exists $p->{alignment};
+	if ( ! defined $p->{borderWidth} && ! defined $default->{borderWidth}) {
+		my $skin = $p->{skin} // $default->{skin} // ( $p->{owner} ? $p->{owner}->skin : '' );
+		$p->{borderWidth} = ($skin eq 'flat') ? 1 : 2;
+	}
 	$self-> SUPER::profile_check_in( $p, $default);
 	@{$p}{qw(selStart selEnd)} = @{$p-> {selection}} if exists( $p-> { selection});
 }
@@ -130,11 +134,20 @@ sub on_paint
 	@selClr = ($self-> hiliteColor, $self-> hiliteBackColor);
 
 	my $border = $self-> {borderWidth};
-	$self-> rect_bevel( $canvas, Prima::rect->new(@size)->inclusive,
-		width  => $self-> {borderWidth},
-		panel  => 1,
-		fill   => $clr[1],
-	);
+	if ( $self-> skin eq 'flat') {
+		$canvas-> rect_fill(
+			Prima::rect->new(@size)->inclusive,
+			$border,
+			$self-> dark3DColor,
+			$clr[1]
+		);
+	} else {
+		$self-> rect_bevel( $canvas, Prima::rect->new(@size)->inclusive,
+			width => $border,
+			panel => 1,
+			fill  => $clr[1],
+		);
+	}
 
 	return if $size[0] <= $border * 2 + 2;
 	my $cap = $self-> {glyphs} or return;
@@ -1280,8 +1293,8 @@ Default value: 2
 
 =item charOffset INTEGER
 
-Selects the position of the cursor in clusters starting from
-the beginning of visual text.
+charOffset gets the current position of the cursor
+charOffset N sets the cursor at position N
 
 =item firstChar
 
