@@ -35,9 +35,10 @@ sub profile_default
 	my %prf = (
 		antialias        => 1,
 		colored          => 1,
+		colorset         => \@warpColors,
 		firstTab         => 0,
 		focusedTab       => 0,
-		height           => $font-> { height} > 14 ? $font-> { height} * 2 : 28,
+		height           => $font-> { height}*2 + $::application->uiScaling * DefGapY,
 		lineWidth        => 1,
 		ownerBackColor   => 1,
 		selectable       => 1,
@@ -59,14 +60,14 @@ sub init
 	for ( qw( colored firstTab focusedTab topMost lastTab arrows)) { $self-> {$_} = 0; }
 	$self-> {tabs}     = [];
 	$self-> {widths}   = [];
+	$self-> {colorset} = [cl::Back];
 	my %profile = $self-> SUPER::init(@_);
-	for ( qw( colored topMost tabs focusedTab firstTab tabIndex)) { $self-> $_( $profile{ $_}); }
+	for ( qw( colorset colored topMost tabs focusedTab firstTab tabIndex)) { $self-> $_( $profile{ $_}); }
 	$self-> recalc_widths;
 	$self-> reset;
 	return %profile;
 }
 
-
 sub reset
 {
 	my $self = $_[0];
@@ -266,6 +267,14 @@ sub on_keydown
 	}
 }
 
+sub skin
+{
+	return $_[0]->SUPER::skin unless $#_;
+	my $self = shift;
+	$self->SUPER::skin($_[1]);
+	$self->repaint;
+}
+
 sub on_paint
 {
 	my ($self,$canvas) = @_;
@@ -277,6 +286,7 @@ sub on_paint
 	}
 	my @c3d  = ( $self-> dark3DColor, $self-> light3DColor);
 	my @size = $canvas-> size;
+	my $flat = $self-> skin eq 'flat';
 
 	$canvas-> color( $clr[1]);
 	$canvas-> bar( 0, 0, @size);
@@ -306,10 +316,10 @@ sub on_paint
 		$atX -= $$ww[$i] + $s * DefGapX;
 		$atXti = $atX, next if $i == $ti;
 		my @poly = (
-			$atX, $s * DefGapY,
-			$atX + $s * DefGapX, $size[1] - $s * DefGapY - 1,
-			$atX + $s * DefGapX + $$ww[$i], $size[1] - $s * DefGapY - 1,
-			$atX + $s * DefGapX * 2 + $$ww[$i], $s * DefGapY
+			$atX, $s * DefGapY + .5,
+			$atX + $s * DefGapX,                $size[1] - $s * DefGapY - 0.5,
+			$atX + $s * DefGapX + $$ww[$i],     $size[1] - $s * DefGapY - 0.5,
+			$atX + $s * DefGapX * 2 + $$ww[$i], $s * DefGapY + .5
 		);
 		@poly[1,3,5,7] = @poly[3,1,7,5] unless $tm;
 		$notifier-> ( @notifyParms, $canvas, $i, \@colorSet, \@poly);
@@ -323,10 +333,10 @@ PaintEarsThen:
 	if ( $ar & 1) {
 		my $x = $s * DefLeftX;
 		my @poly = (
-			$x, $s * DefGapY,
-			$x + $s * DefGapX, $size[1] - $s * DefGapY - 1,
-			$x + $s * DefGapX + $s * DefArrowX, $size[1] - $s * DefGapY - 1,
-			$x + $s * DefGapX * 2 + $s * DefArrowX, $s * DefGapY
+			$x,  $s * DefGapY + .5,
+			$x + $s * DefGapX,                      $size[1] - $s * DefGapY - 0.5,
+			$x + $s * DefGapX + $s * DefArrowX,     $size[1] - $s * DefGapY - 0.5,
+			$x + $s * DefGapX * 2 + $s * DefArrowX, $s * DefGapY + .5
 		);
 		@poly[1,3,5,7] = @poly[3,1,7,5] unless $tm;
 		$notifier-> ( @notifyParms, $canvas, -1, \@colorSet, \@poly);
@@ -334,26 +344,25 @@ PaintEarsThen:
 	if ( $ar & 2) {
 		my $x = $size[0] - $s * (DefLeftX + DefArrowX + DefGapX * 2);
 		my @poly = (
-			$x, $s * DefGapY,
-			$x + $s * DefGapX, $size[1] - $s * DefGapY - 1,
-			$x + $s * (DefGapX + DefArrowX), $size[1] - $s * DefGapY - 1,
-			$x + $s * (DefGapX * 2 + DefArrowX), $s * DefGapY
+			$x,  $s * DefGapY + .5,
+			$x + $s * DefGapX,                   $size[1] - $s * DefGapY - 0.5,
+			$x + $s * (DefGapX + DefArrowX),     $size[1] - $s * DefGapY - 0.5,
+			$x + $s * (DefGapX * 2 + DefArrowX), $s * DefGapY + .5
 		);
 		@poly[1,3,5,7] = @poly[3,1,7,5] unless $tm;
 		$notifier-> ( @notifyParms, $canvas, -2, \@colorSet, \@poly);
 	}
-
 	$canvas-> color( $c3d[0]);
-	my @ld = $tm ? ( 0, $s * DefGapY) : ( $size[1] - 0, $size[1] - $s * DefGapY - 1);
-	$canvas-> line( $size[0] - 1, $ld[0], $size[0] - 1, $ld[1]);
+	my @ld = $tm ? ( -0.5, $s * DefGapY - 0.5) : ( $size[1] - 0, $size[1] - $s * DefGapY - 0.5);
+	$canvas-> line( $size[0] - 0.5, $ld[0], $size[0] - 0.5, $ld[1]);
 
 	if ($tm) {
-		$canvas-> color( $c3d[1]);
+		$canvas-> color( $c3d[1]) unless $flat;
 		$canvas-> line( 0, $ld[1], $size[0] - 1, $ld[1]);
 		$canvas-> line( 0, $ld[0], 0, $ld[1]);
 	} else {
 		$canvas-> line( 0, $ld[1], $size[0] - 1, $ld[1]);
-		$canvas-> color( $c3d[1]);
+		$canvas-> color( $c3d[1]) unless $flat;
 		$canvas-> line( 0, $ld[0], 0, $ld[1]);
 	}
 
@@ -364,21 +373,21 @@ PaintEarsThen:
 PaintSelTabBefore:
 	if ( defined $atXti) {
 		my @poly = (
-			$atXti, $s * DefGapY,
-			$atXti + $s * DefGapX, $size[1] - $s * DefGapY - 1,
-			$atXti + $s * DefGapX + $$ww[$ti], $size[1] - $s * DefGapY - 1,
-			$atXti + $s * DefGapX * 2 + $$ww[$ti], $s * DefGapY
+			$atXti, $s * DefGapY - .5,
+			$atXti + $s * DefGapX,                 $size[1] - $s * DefGapY - 0.5,
+			$atXti + $s * DefGapX + $$ww[$ti],     $size[1] - $s * DefGapY - 0.5,
+			$atXti + $s * DefGapX * 2 + $$ww[$ti], $s * DefGapY - .5
 		);
 		@poly[1,3,5,7] = @poly[3,1,7,5] unless $tm;
 
 		my @poly2 = $tm ? (
-			$atXti, $s * DefGapY,
-			$atXti + $s * DefGapX * 2 + $$ww[$ti], $s * DefGapY,
-			$atXti + $s * DefGapX * 2 + $$ww[$ti] - 4, 0,
-			$atXti + 4, 0
+			$atXti, $s * DefGapY + .5,
+			$atXti + $s * DefGapX * 2 + $$ww[$ti], $s * DefGapY + .5,
+			$atXti + $s * DefGapX * 2 + $$ww[$ti] - 4, 0.5,
+			$atXti + 4, 0.5
 		) : (
-			$atXti, $size[1] - 1 - $s * DefGapY,
-			$atXti + $s * DefGapX * 2 + $$ww[$ti], $size[1] - 1 - $s * DefGapY,
+			$atXti, $size[1] - 1.5 - $s * DefGapY,
+			$atXti + $s * DefGapX * 2 + $$ww[$ti], $size[1] - 1.5 - $s * DefGapY,
 			$atXti + DefGapX * 2 + $$ww[$ti] - 4, $size[1]-1,
 			$atXti + 4, $size[1]-1
 		);
@@ -434,17 +443,21 @@ sub on_drawtab
 {
 	my ( $self, $canvas, $i, $clr, $poly, $poly2) = @_;
 
-	my $color = ( $self-> {colored} && ( $i >= 0)) ?
-		( $warpColors[ $i % scalar @warpColors]) : $$clr[1];
+	my $flat = $self->skin eq 'flat';
+	my $colorset = $self->{colorset};
+	my $color = ( $self-> {colored} && !$flat && ( $i >= 0)) ?
+		( $colorset->[ $i % scalar @$colorset]) : $$clr[1];
 	$color = $self-> prelight_color($color) if ($self->{prelight} // '') eq ($i // '');
 	$canvas-> color($color);
 	$canvas-> fillpoly( $poly);
 	$canvas-> fillpoly( $poly2) if $poly2;
-	$canvas-> color( $$clr[3]);
+	$canvas-> color( $flat ? cl::Black : $$clr[3]);
 	$canvas-> polyline( [@{$poly}[0..($self-> {topMost}?5:3)]]);
-	$canvas-> color( $$clr[2]);
+	if ( !$flat ) {
+		$canvas-> color( $$clr[2]);
+		$canvas-> line( $$poly[4]+1, $$poly[5], $$poly[6]+1, $$poly[7]);
+	}
 	$canvas-> polyline( [@{$poly}[($self-> {topMost}?4:2)..7]]);
-	$canvas-> line( $$poly[4]+1, $$poly[5], $$poly[6]+1, $$poly[7]);
 	$canvas-> color( $$clr[0]);
 	my $s = $::application-> uiScaling;
 
@@ -454,9 +467,12 @@ sub on_drawtab
 			$$poly[1] + ( $$poly[3] - $$poly[1] - $canvas-> font-> height) / 2
 		);
 		$canvas-> text_shape_out( $self-> {tabs}-> [$i], @tx);
-		$canvas-> rect_focus( $tx[0] - 1, $tx[1] - 1,
-			$tx[0] + $self-> {widths}-> [$i] - $s * DefGapX * 4 + 1, $tx[1] + $canvas-> font-> height + 1)
-				if ( $i == $self-> {focusedTab}) && $self-> focused;
+		$canvas-> rect_focus(
+			$tx[0] - 1,
+			$tx[1] - 1,
+			$tx[0] + $self-> {widths}-> [$i] - $s * DefGapX * 4 + 1,
+			$tx[1] + $canvas-> font-> height + 1
+		) if ( $i == $self-> {focusedTab}) && $self-> focused;
 	} elsif ( $i == -1) {
 		$canvas-> fillpoly([
 			$$poly[0] + ( $$poly[6] - $$poly[0]) * 0.6,
@@ -661,7 +677,8 @@ sub set_top_most
 	$self-> repaint;
 }
 
-sub colored      {($#_)?($_[0]-> {colored}=$_[1],$_[0]-> repaint)         :return $_[0]-> {colored}}
+sub colored      {($#_)?($_[0]-> {colored}=$_[1],$_[0]-> repaint)        :return $_[0]-> {colored}}
+sub colorset     {($#_)?($_[0]-> {colorset}=$_[1],$_[0]-> repaint)       :return $_[0]-> {colorset}}
 sub focusedTab   {($#_)?($_[0]-> set_focused_tab(    $_[1]))             :return $_[0]-> {focusedTab}}
 sub firstTab     {($#_)?($_[0]-> set_first_tab(    $_[1]))               :return $_[0]-> {firstTab}}
 sub tabIndex     {($#_)?($_[0]-> set_tab_index(    $_[1]))               :return $_[0]-> {tabIndex}}
@@ -1001,7 +1018,7 @@ use constant Top    => 0;
 use constant Bottom => 1;
 
 package Prima::TabbedNotebook;
-use vars qw(@ISA %notebookProps);
+use vars qw(@ISA %notebookProps %tabsetProps);
 @ISA = qw(Prima::Widget);
 
 use constant DefBorderX   => 11;
@@ -1020,10 +1037,19 @@ for ( keys %notebookProps) {
 GENPROC
 }
 
+%tabsetProps = map { $_, 1 } qw(colored colorset);
+for ( keys %tabsetProps) {
+	eval <<GENPROC;
+   sub $_ { return shift-> {tabSet}-> $_(\@_); }
+GENPROC
+}
+
 sub profile_default
 {
+	my $tabset = Prima::TabSet->profile_default;
 	return {
 		%{Prima::Notebook-> profile_default},
+		(map { $_, $tabset->{$_} } keys %tabsetProps),
 		%{$_[ 0]-> SUPER::profile_default},
 		ownerBackColor      => 1,
 		tabs                => [],
@@ -1054,7 +1080,6 @@ sub init
 	%profile = $self-> SUPER::init(%profile);
 
 	my @size = $self-> size;
-	my $maxh = $self-> font-> height * 2;
 
 	$self-> {tabSet} = $profile{tabsetClass}-> create(
 		owner         => $self,
@@ -1063,7 +1088,7 @@ sub init
 		width         => $size[0],
 		top           => $size[1] - 1,
 		growMode      => gm::Ceiling,
-		height        => $maxh > 28 ? $maxh : 28,
+		(map { $_     => $profile{$_}} keys %tabsetProps),
 		buffered      => 1,
 		designScale   => undef,
 		delegations   => $profile{tabsetDelegations},
@@ -1105,6 +1130,7 @@ sub on_paint
 	my $on_top = ($self-> {orientation} == tno::Top);
 	$canvas-> color( $clr[1]);
 	$canvas-> bar( 0, 0, @size);
+	my $flat = $self->skin eq 'flat';
 
 	my $s = $::application-> uiScaling;
 	if ($self-> {style} == tns::Standard) {
@@ -1114,21 +1140,34 @@ sub on_paint
 			$size[1] -= 5;
 		}
 
-		$canvas-> rect3d(
-			0, 0, $size[0] - 1, $size[1] - 1 + $s * Prima::TabSet::DefGapY,
-			1, reverse @c3d
-		);
-		$canvas-> rect3d(
-			$s * DefBorderX, $on_top ?
-				$s * DefBorderX : $self-> {notebook}-> bottom - 1,
-			$size[0] - 1 - $s * DefBorderX,
-			$size[1] - $s * DefBorderX + $s * Prima::TabSet::DefGapY,
-			1, @c3d
-		);
+		if ( $flat ) {
+			$canvas-> color( $c3d[0] );
+			$canvas-> rectangle(
+				0, 0, $size[0] - 1, $size[1] - 1 + $s * Prima::TabSet::DefGapY,
+			);
+			$canvas-> rectangle(
+				$s * DefBorderX-1, $on_top ?
+					$s * DefBorderX - 1 : $self-> {notebook}-> bottom - 2,
+				$size[0] - 1 - $s * DefBorderX,
+				$size[1] - $s * DefBorderX + $s * Prima::TabSet::DefGapY,
+			);
+		} else {
+			$canvas-> rect3d(
+				0, 0, $size[0] - 1, $size[1] - 1 + $s * Prima::TabSet::DefGapY,
+				1, reverse @c3d
+			);
+			$canvas-> rect3d(
+				$s * DefBorderX, $on_top ?
+					$s * DefBorderX : $self-> {notebook}-> bottom - 1,
+				$size[0] - 1 - $s * DefBorderX,
+				$size[1] - $s * DefBorderX + $s * Prima::TabSet::DefGapY,
+				1, @c3d
+			);
+		}
 
-		my $y = $size[1] - $s * DefBorderX + $s * Prima::TabSet::DefGapY;
-		my $x = $size[0] - $s * DefBorderX - $s * DefBookmarkX;
-		return if $y < $s * DefBorderX * 2 + $s * DefBookmarkX;
+		my $y = int($size[1] - $s * DefBorderX + $s * Prima::TabSet::DefGapY);
+		my $x = int($size[0] - $s * DefBorderX - $s * DefBookmarkX);
+		return if $y < int($s * DefBorderX * 2 + $s * DefBookmarkX);
 
 		my $ar  = 0;
 		my ($pi, $mpi) = (
@@ -1155,7 +1194,6 @@ sub on_paint
 			}
 		}
 		my $fh = $canvas-> font-> height + 8;
-
 		if ( $size[0] - $s * 2 * DefBorderX - $s * DefBookmarkX - 10 > 0 ) {
 			$canvas-> color( $c3d[0]);
 			$canvas-> line(
@@ -1183,14 +1221,14 @@ sub on_paint
 			$canvas-> line( $x - 0, $y - 2, $x + $s * DefBookmarkX - 4, $y - $s * DefBookmarkX + 2);
 			$canvas-> line( $x + 5, $y - $s * DefBookmarkX - 2, $x + $s * DefBookmarkX - 5, $y - $s * DefBookmarkX - 2);
 
-			$canvas-> polyline([
-				$x + $s * 4, $y - $s * DefBookmarkX + $s * 6,
-				$x + $s * 10, $y - $s * DefBookmarkX + $s * 6,
-				$x + $s * 10, $y - $s * DefBookmarkX + $s * 8]) if $ar & 1;
-
 			my $S = int($s);
 			my $dx = $s * DefBookmarkX / 2;
 			my ( $x1, $y1) = ( $x + $dx, $y - $dx);
+			$canvas-> polyline([
+				$x + $s * 4,  $y - $s * DefBookmarkX + $s * 6,
+				$x + $s * 10, $y - $s * DefBookmarkX + $s * 6,
+				$x + $s * 10, $y1 - $s * 8]) if $ar & 1;
+
 			if ( $ar & 2 ) {
 				$canvas-> line( $x1 + $S * 1, $y1 + $S * 4, $x1 + $S * 3, $y1 + $S * 4);
 				$canvas-> line( $x1 + $S * 5, $y1 + $S * 6, $x1 + $S * 5, $y1 + $S * 8);
@@ -1199,8 +1237,16 @@ sub on_paint
 			}
 			$canvas-> color( $c3d[1]);
 			$canvas-> line( $x - 1, $y - 7, $x + $s * DefBookmarkX - 9, $y - $s * DefBookmarkX + 1);
-			$canvas-> line( $s * DefBorderX + 4, $y - $fh * 1.6 - 1, $x - $s * 6, $y - $fh * 1.6 - 1);
-			$canvas-> polyline([ $x + $s * 4, $y1 - $s * 9, $x + $s * 4, $y1 - $s * 8, $x + $s * 10, $y1 - $s * 8]) if $ar & 1;
+			if ( $flat ) {
+				$canvas-> color( $c3d[0]);
+			} else {
+				$canvas-> line( $s * DefBorderX + 4, $y - $fh * 1.6 - 1, $x - $s * 6, $y - $fh * 1.6 - 1);
+			}
+			$canvas-> polyline([
+				$x + $s * 4,  $y - $s * DefBookmarkX + $s * 6,
+				$x + $s * 4,  $y1 - $s * 8,
+				$x + $s * 10, $y1 - $s * 8
+			]) if $ar & 1;
 			if ( $ar & 2 ) {
 				$canvas-> line( $x1 + $S * 3, $y1 + $S * 2, $x1 + $S * 3, $y1 + $S * 3);
 				$canvas-> line( $x1 + $S * 6, $y1 + $S * 6, $x1 + $S * 7, $y1 + $S * 6);
@@ -1228,7 +1274,12 @@ sub on_paint
 		}
 	} else {
 		# tns::Simple
-		$canvas-> rect3d(0, 0, $size[0]-1, $size[1]-1, 1, reverse @c3d);
+		if ( $flat ) {
+			$canvas-> color($c3d[0]);
+			$canvas-> rectangle(0, 0, $size[0]-1, $size[1]-1);
+		} else {
+			$canvas-> rect3d(0, 0, $size[0]-1, $size[1]-1, 1, reverse @c3d);
+		}
 	}
 }
 
@@ -1873,6 +1924,12 @@ A boolean property, selects whether each tab uses unique color
 
 Default value: 1
 
+=item colorset ARRAY
+
+Allows to specify custom colors for the tabs.
+
+Used only when C<colored> is set to 1.
+
 =item firstTab INTEGER
 
 Selects the first ( leftmost ) visible tab.
@@ -1985,6 +2042,9 @@ First-level tab is often referred as I<tab>, and second-level as I<page>.
 
 =head2 Properties
 
+The class forwards the following properties of C<Prima::TabSet>, which are
+described in L<Prima::TabSet>: C<colored>, C<colorset>
+
 =over
 
 =item defaultInsertPage INTEGER
