@@ -32,7 +32,7 @@ sub profile_default
 		color          => cl::Black,
 		selectable     => 0,
 		tabStop        => 0,
-		widgetClass    => wc::Button,
+		widgetClass    => wc::ScrollBar,
 	}
 }
 
@@ -192,12 +192,22 @@ sub on_paint
 	my @size = $canvas-> size;
 	my $p = $self-> {pressState};
 
-	$canvas-> rect3d( 0, 0, $size[0] - 1, $size[1] * 0.4 - 1, 2,
-		(($p != 2) ? @c3d : reverse @c3d), ($prelightPart eq 'upper') ? $prelightColor : $clr[1]);
-	$canvas-> rect3d( 0, $size[1] * 0.4, $size[0] - 1, $size[1] * 0.6 - 1, 2,
-		(($p != 3) ? @c3d : reverse @c3d), ($prelightPart eq 'middle') ? $prelightColor : $clr[1]);
-	$canvas-> rect3d( 0, $size[1] * 0.6, $size[0] - 1, $size[1] - 1, 2,
-		(($p != 1) ? @c3d : reverse @c3d), ($prelightPart eq 'lower') ? $prelightColor : $clr[1]);
+	my @parts = (
+		lower  => [0, $size[1] * 0.6, $size[0] - 1, $size[1] - 1],
+		upper  => [0, 0, $size[0] - 1, $size[1] * 0.4 - 1],
+		middle => [0, $size[1] * 0.4, $size[0] - 1, $size[1] * 0.6 - 1],
+	);
+	my $flat = $self-> skin eq 'flat';
+	for ( my ($i, $pressed) = (0, 1); $i < @parts; $i+=2, $pressed++) {
+		my $clr  = ($prelightPart eq $parts[$i]) ? $prelightColor : $clr[1];
+		my $rect = $parts[$i+1];
+		if ( $flat ) {
+			$self-> color($clr);
+			$self-> bar( @$rect );
+		} else {
+			$canvas-> rect3d( @$rect, 2, (($p != $pressed) ? @c3d : reverse @c3d), $clr);
+		}
+	}
 
 	$canvas-> color( $clr[0]);
 	my $p1 = ( $p == 1) ? 1 : 0;
@@ -233,6 +243,7 @@ sub profile_default
 		%{$_[ 0]-> SUPER::profile_default},
 		width        => 18 * $::application-> uiScaling,
 		height       => 18 * $::application-> uiScaling,
+		flatSeparator => 0,
 	}
 }
 
@@ -244,6 +255,14 @@ sub profile_check_in
 	$self-> SUPER::profile_check_in( $p, $default);
 }
 
+sub init
+{
+	my $self = shift;
+	my %profile = $self-> SUPER::init( @_);
+	$self-> {flatSeparator} = $profile{flatSeparator};
+	return %profile;
+}
+
 sub on_mousedown
 {
 	my ( $self, $btn, $mod, $x, $y) = @_;
@@ -329,7 +348,6 @@ sub fix_triangle
 	return \@spot;
 }
 
-
 sub on_paint
 {
 	my ( $self, $canvas) = @_;
@@ -337,6 +355,7 @@ sub on_paint
 	@clr = ( $self-> hiliteColor, $self-> hiliteBackColor)     if $self-> { default};
 	@clr = ( $self-> disabledColor, $self-> disabledBackColor) if !$self-> enabled;
 	my ($prelightPart, $prelightColor) = (0);
+	my $flat = $self->skin eq 'flat';
 	if ($self->{prelight}) {
 		$prelightColor = $self-> prelight_color($clr[1], 1.5);
 		$prelightPart  = $self->{prelight};
@@ -355,10 +374,18 @@ sub on_paint
 			$size[0] - 3, $size[1] - 3,
 		]);
 	}
-	$canvas-> color( $p == 1 ? 0x404040 : $c3d[1]);
-	$canvas-> polyline( [0, 0, 0, $size[1] - 1, $size[0] - 2, $size[1] - 1]);
-	$canvas-> color( $p == 1 ? $c3d[1]  : $c3d[0]);
-	$canvas-> polyline( [1, 1, 1, $size[1] - 2, $size[0] - 3, $size[1] - 2]);
+
+	if ( $flat ) {
+		if ( $self->{flatSeparator}) {
+			$canvas-> color( $self-> dark3DColor );
+			$canvas-> line(0, 0, 0, $size[1]);
+		}
+	} else {
+		$canvas-> color( $p == 1 ? 0x404040 : $c3d[1]);
+		$canvas-> polyline( [0, 0, 0, $size[1] - 1, $size[0] - 2, $size[1] - 1]);
+		$canvas-> color( $p == 1 ? $c3d[1]  : $c3d[0]);
+		$canvas-> polyline( [1, 1, 1, $size[1] - 2, $size[0] - 3, $size[1] - 2]);
+	}
 
 	if ( $prelightPart == 2 && $size[1] > 4 && $size[0] > 4 ) {
 		$canvas->color( $prelightColor );
@@ -368,17 +395,19 @@ sub on_paint
 			$size[0] - 3, 2,
 		]);
 	}
-	$canvas-> color( $p == 2 ? $c3d[0] : $c3d[1]);
-	$canvas-> polyline([2, 1, $size[0] - 2, 1, $size[0] - 2, $size[1] - 2]);
-	$canvas-> color( $p == 2 ? $c3d[1] : 0x404040);
-	$canvas-> polyline([1, 0, $size[0] - 1, 0, $size[0] - 1, $size[1] - 1]);
+	unless ( $flat ) {
+		$canvas-> color( $p == 2 ? $c3d[0] : $c3d[1]);
+		$canvas-> polyline([2, 1, $size[0] - 2, 1, $size[0] - 2, $size[1] - 2]);
+		$canvas-> color( $p == 2 ? $c3d[1] : 0x404040);
+		$canvas-> polyline([1, 0, $size[0] - 1, 0, $size[0] - 1, $size[1] - 1]);
 
-	$canvas-> color( $p == 1 ? $c3d[ 0] : $c3d[ 1]);
-	$canvas-> line( -1, 0, $size[0] - 2, $size[1] - 1);
-	$canvas-> color( 0x404040);
-	$canvas-> line( 0, 0, $size[0] - 1, $size[1] - 1);
-	$canvas-> color( $p == 2 ? $c3d[ 1] : $c3d[ 0]);
-	$canvas-> line( 1, 0, $size[0], $size[1] - 1);
+		$canvas-> color( $p == 1 ? $c3d[ 0] : $c3d[ 1]);
+		$canvas-> line( -1, 0, $size[0] - 2, $size[1] - 1);
+		$canvas-> color( 0x404040);
+		$canvas-> line( 0, 0, $size[0] - 1, $size[1] - 1);
+		$canvas-> color( $p == 2 ? $c3d[ 1] : $c3d[ 0]);
+		$canvas-> line( 1, 0, $size[0], $size[1] - 1);
+	}
 
 	$canvas-> color( $clr[0]);
 	my $p1 = ( $p == 1) ? 1 : 0;
@@ -404,6 +433,8 @@ sub set_state
 	$self-> repaint;
 }
 
+sub flatSeparator { $#_ ? $_[0]->{flatSeparator} = $_[1] : $_[0]->{flatSeparator} }
+
 package Prima::SpinEdit;
 use vars qw(@ISA %editProps %spinDynas);
 use Prima::InputLine;
@@ -436,6 +467,7 @@ sub profile_default
 		%{Prima::InputLine-> profile_default},
 		%{$_[ 0]-> SUPER::profile_default},
 		autoEnableChildren => 1,
+		allowEmpty     => 0,
 		ownerBackColor => 1,
 		selectable     => 0,
 		scaleChildren  => 0,
@@ -467,11 +499,13 @@ sub init
 	my ( $w, $h) = ( $self-> size);
 	$self-> {spin} = $self-> insert( $profile{spinClass} =>
 		ownerBackColor => 1,
+		ownerColor     => 1,
 		name           => 'Spin',
 		bottom         => 1,
 		right          => $w - 1,
 		height         => $h - 1 * 2,
 		growMode       => gm::Right,
+		flatSeparator  => 1,
 		delegations    => $profile{spinDelegations},
 		(map { $_ => $profile{$_}} grep { exists $profile{$_} ? 1 : 0} keys %spinDynas),
 		%{$profile{spinProfile}},
@@ -490,7 +524,7 @@ sub init
 		%{$profile{editProfile}},
 		text        => $profile{value},
 	);
-	for (qw( min max step value circulate pageStep)) {$self-> $_($profile{$_});};
+	for (qw( allowEmpty min max step value circulate pageStep)) {$self-> $_($profile{$_});};
 	$self-> visible( $visible);
 	return %profile;
 }
@@ -499,7 +533,12 @@ sub on_paint
 {
 	my ( $self, $canvas) = @_;
 	my @s = $canvas-> size;
-	$canvas-> rect3d( 0, 0, $s[0]-1, $s[1]-1, 1, $self-> dark3DColor, $self-> light3DColor);
+	if ( $self-> skin eq 'flat') {
+		$canvas->color( $self->dark3DColor);
+		$canvas->rectangle(0,0,$s[0]-1,$s[1]-1);
+	} else {
+		$canvas-> rect3d( 0, 0, $s[0]-1, $s[1]-1, 1, $self-> dark3DColor, $self-> light3DColor);
+	}
 }
 
 sub InputLine_MouseWheel
@@ -508,6 +547,7 @@ sub InputLine_MouseWheel
 	$z = (abs($z) > 120) ? int($z/120) : (($z > 0) ? 1 : -1);
 	$z *= $self-> {pageStep} if $mod & km::Ctrl;
 	my $value = $self-> value;
+	$value = $self->min if $value eq '';
 	$self-> value( $value + $z * $self-> {step});
 	$self-> value( $z > 0 ? $self-> min : $self-> max)
 		if $self-> {circulate} && ( $self-> value == $value);
@@ -522,7 +562,9 @@ sub InputLine_DragEnd
 	return unless defined $text;
 	$text =~ s/^\s+//;
 	$text =~ s/\s+$//;
-	return if $text =~ /^-?\d+(\.\d+)?$/ and $text >= $self->min and $text <= $self->max;
+	my $ok = $text =~ /^-?\d+(\.\d+)?$/ && $text >= $self->min && $text <= $self->max;
+	$ok = 1 if $self->{allowEmpty} && $text eq '';
+	return unless $ok;
 	$edit->clear_event;
 	$edit->on_dragend(undef, $action, $mod, $x, $y, $ref);
 	$ref->{allow} = 0;
@@ -532,6 +574,7 @@ sub Spin_Increment
 {
 	my ( $self, $spin, $increment) = @_;
 	my $value = $self-> value;
+	$value = $self->min if $value eq '';
 	$self-> value( $value + $increment * $self-> {step});
 	$self-> value( $increment > 0 ? $self-> min : $self-> max)
 		if $self-> {circulate} && ( $self-> value == $value);
@@ -551,6 +594,7 @@ sub InputLine_KeyDown
 			$self-> value( $key == kb::PgDn ? $self-> min : $self-> max);
 		} else {
 			my $value = $self-> value;
+			$value = $self->min if $value eq '';
 			$self-> value( $value + $z);
 			$self-> value( $z > 0 ? $self-> min : $self-> max)
 				if $self-> {circulate} && ( $self-> value == $value);
@@ -589,7 +633,7 @@ sub set_bounds
 	my ( $self, $min, $max) = @_;
 	$max = $min if $max < $min;
 	( $self-> { min}, $self-> { max}) = ( $min, $max);
-	my $oldValue = $self-> value;
+	return if $self->{allowEmpty} && $self->value eq '';
 	$self-> value( $max) if $max < $self-> value;
 	$self-> value( $min) if $min > $self-> value;
 }
@@ -613,6 +657,7 @@ sub pageStep
 	$_[0]-> {pageStep} = $_[1];
 }
 
+sub allowEmpty   {$#_ ? $_[0]-> {allowEmpty} = $_[1] : return $_[0]-> {allowEmpty}}
 
 sub min          {($#_)?$_[0]-> set_bounds($_[1], $_[0]-> {'max'})      : return $_[0]-> {min};}
 sub max          {($#_)?$_[0]-> set_bounds($_[0]-> {'min'}, $_[1])      : return $_[0]-> {max};}
@@ -621,7 +666,10 @@ sub value
 {
 	if ($#_) {
 		my ( $self, $value) = @_;
-		if ( $value =~ m/^\s*([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?\s*$/) {
+		$value //= '';
+		if ( $self->{allowEmpty} && $value eq '') {
+			# ok
+		} elsif ( $value =~ m/^\s*([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?\s*$/) {
 			$value = $self-> {min} if $value < $self-> {min};
 			$value = $self-> {max} if $value > $self-> {max};
 		} else {
@@ -632,7 +680,9 @@ sub value
 	} else {
 		my $self = $_[0];
 		my $value = $self-> {edit}-> text;
-		if ( $value =~ m/^\s*([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?\s*$/) {
+		if ( $self->{allowEmpty} && $value eq '') {
+			# ok
+		} elsif ( $value =~ m/^\s*([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?\s*$/) {
 			$value = $self-> {min} if $value < $self-> {min};
 			$value = $self-> {max} if $value > $self-> {max};
 		} else {
@@ -725,7 +775,7 @@ sub on_paint
 	my ( $l3, $d3) = ( $self-> light3DColor, $self-> dark3DColor);
 	$canvas-> color( $clComplete);
 
-	# INDETERMINATE STYLE HACK
+ 	# INDETERMINATE STYLE HACK
 	my $left_bound =
 		$self->indeterminate ?
 			$complete - ($self->{sliderLength} * ($v ? $y : $x) / $range + 0.5) :
@@ -1189,6 +1239,7 @@ sub on_paint
 	my @cht  = ( $self-> hiliteColor, $self-> hiliteBackColor);
 	my @glyph_deltas = ([$clr[0], 0, 0]);
 	unshift @glyph_deltas, [cl::White, 1, -1] unless $enabled;
+	my $flat = $self-> skin eq 'flat';
 
 	my @size = $canvas-> size;
 	my (
@@ -1249,7 +1300,7 @@ sub on_paint
 	}
 
 	if ( $v) {
-		my $bw  = ( $size[0] - $sb) / 2;
+		my $bw  = int(( $size[0] - $sb) / 2 + .5);
 		return if $size[1] <= $kb * ($self-> {readOnly} ? 1 : 0) + 2 * $bh + 2;
 
 		$canvas-> translate((( $ta == 1) ? 1 : -1) * ( $bw - $sb - $kb), 0)
@@ -1337,22 +1388,29 @@ sub on_paint
 			my $rgn = Prima::Region->new( polygon => \@jp);
 			$rgn->offset( $canvas->translate );
 			$canvas-> region( $rgn );
-			$canvas-> new_gradient(
-				palette  => [ $c3d[0], ($self->{prelight} ? $prelight : $clr[1]) ],
-				poly     => [0,0,0.3,0.7,1,1],
-				vertical => 0,
-			)-> bar( $jp[0]+2,$jp[1]+2,$jp[6]-2,$jp[3]-2);
-			$canvas-> color( 0x404040);
-			$canvas-> polyline([@jp[6..9,0,1]]);
-			$canvas-> color( $c3d[1]);
-			$canvas-> polyline([$jp[0]+1,$jp[1]+1,$jp[2]+1,$jp[3]-1,$jp[4],$jp[5]-1,$jp[6]-1,$jp[7]]);
-			$canvas-> line($jp[0]+2, $jp[7]-1, $jp[6]-2, $jp[7]-1);
-			$canvas-> color( $c3d[0]);
-			$canvas-> polyline([$jp[6]-1,@jp[7,8],$jp[9]+1,$jp[0],$jp[1]+1,@jp[2..7]]);
-			$canvas-> line($jp[0]+2, $jp[7]+1, $jp[6]-1, $jp[7]+1);
+			if ( $flat ) {
+				$canvas-> color( $self->{prelight} ? $prelight : $clr[1]);
+				$canvas-> fillpoly([@jp[0..9,0,1]]);
+				$canvas-> color( 0x404040 );
+				$canvas-> polyline([@jp[0..9,0,1]]);
+			} else {
+				$canvas-> new_gradient(
+					palette  => [ $c3d[0], ($self->{prelight} ? $prelight : $clr[1]) ],
+					poly     => [0,0,0.3,0.7,1,1],
+					vertical => 0,
+				)-> bar( $jp[0]+2,$jp[1]+2,$jp[6]-2,$jp[3]-2);
+				$canvas-> color( 0x404040);
+				$canvas-> polyline([@jp[6..9,0,1]]);
+				$canvas-> color( $c3d[1]);
+				$canvas-> polyline([$jp[0]+1,$jp[1]+1,$jp[2]+1,$jp[3]-1,$jp[4],$jp[5]-1,$jp[6]-1,$jp[7]]);
+				$canvas-> line($jp[0]+2, $jp[7]-1, $jp[6]-2, $jp[7]-1);
+				$canvas-> color( $c3d[0]);
+				$canvas-> polyline([$jp[6]-1,@jp[7,8],$jp[9]+1,$jp[0],$jp[1]+1,@jp[2..7]]);
+				$canvas-> line($jp[0]+2, $jp[7]+1, $jp[6]-1, $jp[7]+1);
+			}
 		}
 	} else {
-		my $bh  = ( $size[1] - $sb) / 2;
+		my $bh  = int(( $size[1] - $sb) / 2 + .5);
 		my $fh = $canvas-> font-> height;
 		return if $size[0] <= $kb * ($self-> {readOnly} ? 1 : 0) + 2 * $bw + 2;
 
@@ -1455,22 +1513,29 @@ sub on_paint
 			my $rgn = Prima::Region->new( polygon => \@jp);
 			$rgn->offset( $canvas->translate );
 			$canvas-> region( $rgn );
-			$canvas-> new_gradient(
-				palette  => [ ($self->{prelight} ? $prelight : $clr[1]), $c3d[0] ],
-				poly     => [0,0,0.7,0.3,1,1],
-				vertical => 1,
-			)-> bar( $jp[0]+2,$jp[9],$jp[4]-2,$jp[3]);
-			$canvas-> color( 0x404040 );
-			$canvas-> polyline([@jp[4..9]]);
-			$canvas-> color( $c3d[0]);
-			$canvas-> polyline([
-				@jp[8,9,0..3],$jp[4]-1,$jp[5],
-				$jp[6]-1,$jp[7],$jp[8],$jp[9]+1
-			]);
-			$canvas-> line($jp[8]-1,$jp[3]-2,$jp[8]-1,$jp[9]) if $kb > 10;
-			$canvas-> color( $c3d[1]);
-			$canvas-> polyline([$jp[8],$jp[9]+1,$jp[0]+1,$jp[1],$jp[2]+1,$jp[3]-1,$jp[4]-2,$jp[5]-1]);
-			$canvas-> line($jp[8]+1,$jp[3]-2,$jp[8]+1,$jp[9]) if $kb > 10;
+			if ( $flat) {
+				$canvas-> color( $self->{prelight} ? $prelight : $clr[1]);
+				$canvas-> fillpoly([ @jp[8,9,0..9] ]);
+				$canvas-> color( 0x404040 );
+				$canvas-> polyline([ @jp[8,9,0..9] ]);
+			} else {
+				$canvas-> new_gradient(
+					palette  => [ ($self->{prelight} ? $prelight : $clr[1]), $c3d[0] ],
+					poly     => [0,0,0.7,0.3,1,1],
+					vertical => 1,
+				)-> bar( $jp[0]+2,$jp[9],$jp[4]-2,$jp[3]);
+				$canvas-> color( 0x404040 );
+				$canvas-> polyline([@jp[4..9]]);
+				$canvas-> color( $c3d[0]);
+				$canvas-> polyline([
+					@jp[8,9,0..3],$jp[4]-1,$jp[5],
+					$jp[6]-1,$jp[7],$jp[8],$jp[9]+1
+				]);
+				$canvas-> line($jp[8]-1,$jp[3]-2,$jp[8]-1,$jp[9]) if $kb > 10;
+				$canvas-> color( $c3d[1]);
+				$canvas-> polyline([$jp[8],$jp[9]+1,$jp[0]+1,$jp[1],$jp[2]+1,$jp[3]-1,$jp[4]-2,$jp[5]-1]);
+				$canvas-> line($jp[8]+1,$jp[3]-2,$jp[8]+1,$jp[9]) if $kb > 10;
+			} 
 		}
 	}
 }
@@ -2059,6 +2124,7 @@ sub on_paint
 	my $br  = $self-> {br};
 	my $rad = $self-> {radius};
 	my @cpt = ( $self-> {circX}, $self-> {circY}, $rad*2+1, $rad*2+1);
+	my $flat = $self->skin eq 'flat';
 
 	goto AFTER_DIAL unless $self->{show_dial};
 	if ( defined $self-> {singlePaint}) {
@@ -2083,6 +2149,7 @@ sub on_paint
 		my $dp   = 2;
 		$canvas-> lineWidth(2);
 		for my $lw (2..4) {
+			next if $flat;
 			$canvas-> color( $c3d[1]);
 			$canvas-> arc( @cpt[0..1], $cpt[2]-$dp, $cpt[3]-$dp, 65 + $da, 235 - $da);
 			$canvas-> color( $c3d[0]);
@@ -2114,15 +2181,19 @@ sub on_paint
 		$knob += $rad / 50;
 		my @cxt = ( $self-> offset2pt( @cpt[0,1], $self-> {value}, $rad - $knob * 1.5), 4, 4);
 		$canvas->antialias(1);
-		$canvas-> lineWidth(($rad > 50) ? 2 : 1);
 		$canvas-> color( $c3d[0]);
-		$canvas-> arc( @cxt[0..1], $knob, $knob, 65, 235);
-		$canvas-> color((
-			cl::distance( $canvas->map_color($c3d[1]), $canvas->map_color($clr[1])) > 
-			cl::distance( $canvas->map_color($c3d[1]), 0)) ?
-				$c3d[1] : 0);
-		$canvas-> arc( @cxt[0..1], $knob, $knob, 255, 405);
-		$canvas-> lineWidth(0);
+		if ( $flat ) {
+			$canvas-> fill_ellipse( @cxt[0..1], $knob+1, $knob+1);
+		} else {
+			$canvas-> lineWidth(($rad > 50) ? 2 : 1);
+			$canvas-> arc( @cxt[0..1], $knob, $knob, 65, 235);
+			$canvas-> color((
+				cl::distance( $canvas->map_color($c3d[1]), $canvas->map_color($clr[1])) > 
+				cl::distance( $canvas->map_color($c3d[1]), 0)) ?
+					$c3d[1] : 0);
+			$canvas-> arc( @cxt[0..1], $knob, $knob, 255, 405);
+			$canvas-> lineWidth(0);
+		}
 		$canvas->antialias(0);
 	}
 AFTER_DIAL:
@@ -2765,6 +2836,12 @@ selection properties.
 
 =over
 
+=item allowEmpty BOOLEAN
+
+If set, allows an empty string as a valid value
+
+Default value: false
+
 =item circulate BOOLEAN
 
 Selects the value modification rule when the increment or decrement
