@@ -534,6 +534,7 @@ sub renew_widgets
 	if ( $VB::form) {
 		my @f = ( $VB::form, $VB::form-> widgets);
 		$_ = $_-> name for @f;
+		@f = sort @f;
 		$VB::inspector-> Selector-> items( \@f);
 		my $fx = $VB::form-> focused ? $VB::form : $VB::form-> selectedWidget;
 		$fx = $VB::form unless $fx;
@@ -939,9 +940,7 @@ sub fm_reclass
 	my $self = $VB::form;
 	return unless $self;
 	my @wijs = $VB::form-> marked_widgets;
-	return unless scalar @wijs;
-	my $cx = $wijs[0];
-	$self = $cx if $cx;
+	$self = $wijs[0] if @wijs;
 	my $lab_text = 'Class name';
 	$lab_text =  "Temporary class for ".$self-> {realClass} if defined $self-> {realClass};
 	my $dlg = Prima::Dialog-> create(
