@@ -67,6 +67,7 @@ Widget_init( Handle self, HV * profile)
 	opt_assign( optOwnerFont     , pget_B( ownerFont));
 	opt_assign( optOwnerHint     , pget_B( ownerHint));
 	opt_assign( optOwnerShowHint , pget_B( ownerShowHint));
+	opt_assign( optOwnerSkin     , pget_B( ownerSkin));
 	opt_assign( optOwnerPalette  , pget_B( ownerPalette));
 	my-> colorIndex( self, true,  ciHiliteText,   pget_i( hiliteColor)      );
 	my-> colorIndex( self, true,  ciHilite,       pget_i( hiliteBackColor)  );
@@ -100,6 +101,7 @@ Widget_init( Handle self, HV * profile)
 	my-> set_pointerType        ( self, pget_i( pointerType));
 	my-> set_selectingButtons   ( self, pget_i( selectingButtons));
 	my-> set_selectable         ( self, pget_B( selectable));
+	my-> set_skin               ( self, pget_sv( skin));
 	my-> set_showHint           ( self, pget_B( showHint));
 	my-> set_tabOrder           ( self, pget_i( tabOrder));
 	my-> set_tabStop            ( self, pget_B( tabStop));
@@ -118,12 +120,12 @@ Widget_init( Handle self, HV * profile)
 	my-> popupColorIndex( self, true, ciDark3DColor,  pget_i( popupDark3DColor)        );
 	SvHV_Font( pget_sv( popupFont), &Font_buffer, "Widget::init");
 	my-> set_popup_font  ( self, Font_buffer);
-	if ( SvTYPE( sv = pget_sv( popupItems)) != SVt_NULL)
+	if ( SvOK( sv = pget_sv( popupItems)))
 		my-> set_popupItems( self, sv);
-	if ( SvTYPE( sv = pget_sv( accelItems)) != SVt_NULL)
+	if ( SvOK( sv = pget_sv( accelItems)))
 		my-> set_accelItems( self, sv);
 
-	/* size, position, enabling, visibliity etc. runtime */
+	/* size, position, enabling, visibility etc. runtime */
 	{
 		Point set, set2;
 		AV * av;
@@ -191,7 +193,7 @@ Widget_init( Handle self, HV * profile)
 
 	{
 		SV * widgets = pget_sv( widgets);
-		if ( SvTYPE( widgets) != SVt_NULL) {
+		if ( SvOK( widgets) ) {
 			dSP;
 			ENTER;
 			SAVETMPS;
@@ -259,6 +261,8 @@ Widget_done( Handle self)
 	apc_widget_destroy( self);
 	if ( var-> hint ) sv_free( var->hint);
 	var-> hint = NULL;
+	if ( var-> skin ) sv_free( var->skin);
+	var-> skin = NULL;
 	free( var-> helpContext);
 	var-> helpContext = NULL;
 
@@ -734,6 +738,8 @@ Widget_set( Handle self, HV * profile)
 				my-> set_font ( self, CWidget( postOwner)-> get_font( postOwner));
 				opt_set( optOwnerFont);
 			}
+			if ( is_opt( optOwnerSkin))
+				my-> set_skin( self, NULL_SV);
 		}
 		if ( var-> geometry != gtDefault) {
 			geometry = var-> geometry;
@@ -1091,7 +1097,7 @@ Widget_hintVisible( Handle self, Bool set, int hintVisible)
 	wantVisible = ( hintVisible != 0);
 	if ( wantVisible == P_APPLICATION-> hintVisible) return false;
 	if ( wantVisible) {
-		if ( SvTYPE(var->hint) == SVt_NULL || (SvCUR( var-> hint) == 0)) return false;
+		if ( !SvOK(var->hint) || (SvCUR( var-> hint) == 0)) return false;
 		if ( hintVisible > 0) P_APPLICATION-> hintActive = -1; /* immediate */
 	}
 	C_APPLICATION-> set_hint_action( prima_guts.application, self, wantVisible, false);
@@ -1557,6 +1563,48 @@ Widget_cursorVisible( Handle self, Bool set, Bool cursorVisible)
 	return apc_cursor_set_visible( self, cursorVisible);
 }
 
+static Bool
+propagate_skin( Handle owner, Handle self, void * dummy)
+{
+	enter_method;
+	my-> set_skin( self, var-> skin);
+	return false;
+}
+
+SV *
+Widget_skin( Handle self, Bool set, SV *skin)
+{
+	if ( !set) {
+		return
+			is_opt( optOwnerSkin) ? (
+				var-> owner ?
+					CWidget(var->owner)->get_skin(var->owner) :
+					NULL_SV
+			) : (
+				(var->skin == NULL) ?
+					NULL_SV :
+					newSVsv(var->skin)
+			);
+	} else if ( var-> stage <= csFrozen) {
+		enter_method;
+		if ( var-> skin == skin) {
+			my-> first_that( self, (void*)propagate_skin, NULL);
+			return NULL_SV;
+		}
+		if ( var-> skin )
+			sv_free( var-> skin );
+		if ( SvOK( skin )) {
+			var-> skin = newSVsv(skin);
+			opt_clear( optOwnerSkin);
+		} else if ( !is_opt( optOwnerSkin)) {
+			var-> skin = NULL;
+			opt_set( optOwnerSkin);
+		}
+		my-> first_that( self, (void*)propagate_skin, NULL);
+	}
+	return NULL_SV;
+}
+
 SV *
 Widget_dndAware( Handle self, Bool set, SV * dndAware)
 {
@@ -1660,7 +1708,7 @@ Widget_set_hint( Handle self, SV *hint)
 		P_APPLICATION-> hintUnder == self)
 	{
 		Handle hintWidget = P_APPLICATION-> hintWidget;
-		if ( SvTYPE(var->hint) == SVt_NULL || (SvLEN( var-> hint) == 0))
+		if ( !SvOK(var->hint) || (SvLEN( var-> hint) == 0))
 			my-> set_hintVisible( self, 0);
 		if ( hintWidget)
 			CWidget(hintWidget)-> set_text( hintWidget, my-> get_hint( self));
@@ -1792,6 +1840,30 @@ Widget_ownerShowHint( Handle self, Bool set, Bool ownerShowHint )
 	return false;
 }
 
+Bool
+Widget_ownerSkin( Handle self, Bool set, Bool ownerSkin )
+{
+	Bool before;
+	enter_method;
+	if ( !set)
+		return is_opt( optOwnerSkin);
+
+	before = is_opt( optOwnerSkin );
+	ownerSkin = ownerSkin ? 1 : 0;
+	if ( ownerSkin == before ) return false;
+
+	if ( ownerSkin )
+		my-> set_skin( self, NULL_SV);
+	else if ( var-> owner) {
+		PWidget o = (PWidget) var-> owner;
+		while ( o-> owner != NULL_HANDLE && o->options.optOwnerSkin)
+			o = (PWidget) o->owner;
+		my-> set_skin( self, o-> skin );
+	}
+
+	return false;
+}
+
 SV *
 Widget_palette( Handle self, Bool set, SV * palette)
 {
@@ -2055,7 +2127,7 @@ Widget_selectedWidget( Handle self, Bool set, Handle widget)
 		}
 		return NULL_HANDLE;
 
-		/* classic solution should be recursive and inheritant call */
+		/* classic solution should be recursive and inherited call */
 		/* of get_selected() here, when Widget would return state of */
 		/* child-group selected state until Widget::selected() called; */
 		/* thus, each of them would call apc_widget_get_focused - that's expensive, */
