@@ -134,6 +134,7 @@ sub profile_default
 				['C~opy'  => 'Ctrl+Ins'   => kb::NoKey, sub{$_[0]-> {editor}-> copy}],
 				['~Paste' => 'Shift+Ins'  => kb::NoKey, sub{$_[0]-> {editor}-> paste}],
 				['~Delete' => 'Shift+Del' => kb::NoKey, sub{$_[0]-> {editor}-> delete_block}],
+				['~Select all' => 'Ctrl+A' => '^A' => sub { $_[0]->{editor}->select_all }],
 				[],
 				['~Find...' => 'Esc'      => kb::Esc   , q(find)],
 				['~Replace...'=> 'Ctrl+S' => '^S'      , q(replace)],
@@ -145,7 +146,7 @@ sub profile_default
 			['~Options' => [
 				[ '@syx' => '~Syntax hilite' => sub{ $_[0]-> {editor}-> syntaxHilite( $_[2] )}],
 				[ '@*aid' => '~Auto indent' => sub{ $_[0]-> {editor}-> autoIndent( $_[2] )}],
-				[ '@wwp' => '~Word wrap' => sub{ $_[0]-> {editor}-> wordWrap( $_[2] )}],
+				[ '@*wwp' => '~Word wrap' => sub{ $_[0]-> {editor}-> wordWrap( $_[2] )}],
 				[ '@cwp' => '~Cursor wrap' => sub{ $_[0]-> {editor}-> cursorWrap( $_[2] )}],
 				[ '@tab' => 'Enter ~tabs in text' => sub{ $_[0]-> {editor}-> wantTabs( $_[2] )}],
 				[ '@psb' => '~Persistent blocks' => sub{ $_[0]-> {editor}-> persistentBlock( $_[2] )}],
@@ -191,7 +192,7 @@ sub init
 		}
 	}
 	$fn = '.Untitled' unless defined $fn;
-	my $fh = $::application->font->height + 1;
+	my $fh = $::application->font->height + 2;
 	$self-> {editor} = $self-> insert( Editor =>
 		name      => 'Edit',
 		textRef   => \$cap,
