@@ -1,4 +1,4 @@
-=pod 
+=pod
 
 =head1 NAME
 
@@ -24,7 +24,7 @@ use warnings;
 use Prima;
 use Prima::ImageViewer;
 use Prima::Application;
-use Prima::RubberBand;
+use Prima::Widget::RubberBand;
 use Prima::Drawable::Subcanvas;
 
 package Generic;
@@ -52,6 +52,7 @@ my $w = Prima::MainWindow-> create(
 			$self-> capture( 1);
 			$self-> { cap} = 2;
 		}],
+		[ "~Full screen" => sub { shift-> IV-> image( $::application->get_fullscreen_image) }],
 	],
 	onMouseDown => sub {
 		my ( $self, $btn, $mod, $x, $y) = @_;
