@@ -339,8 +339,10 @@ my $g = $w-> insert( Periodic =>
 			} else {
 				$canvas-> color( $color);
 			}
-			$canvas-> text_out( $item, $cx1 + 10 * $scaling, $cy1 + 10 * $scaling);
 			my $f = $canvas-> font;
+			$f->style(fs::Bold);
+			$canvas-> text_out( $item, $cx1 + 10 * $scaling, $cy1 + 10 * $scaling);
+			$f->style(fs::Normal);
 			$canvas-> font( height => 12 * $scaling );
 				@small_font_metrics = ( $canvas-> get_text_width('3'), $f-> height)
 					unless @small_font_metrics;
