@@ -29,7 +29,6 @@ typedef struct {
 	Handle tile;
 	unsigned char * linePattern;
 	PRegionRec region;
-	Point translate;
 } ImgPaintContext, *PImgPaintContext;
 
 typedef void BitBltProc( Byte * src, Byte * dst, int count);
@@ -49,6 +48,12 @@ extern void init_image_support(void);
 
 /* image basic routines */
 #define LINE_SIZE(width,type) (((( width ) * (( type ) & imBPP) + 31) / 32) * 4)
+#define EFFECTIVE_LINE_SIZE(width,type) (\
+	 ((type&imBPP) == 1) ? (((width) + 7) / 8)  : \
+	(((type&imBPP) == 4) ? (((width) + 1) / 2) : \
+	((width)*(type&imBPP)/8)) \
+)
+
 extern Bool ic_stretch( int type, Byte * srcData, int srcW, int srcH, Byte * dstData, int w, int h, int scaling, char * error);
 extern int  ic_stretch_suggest_type( int type, int scaling );
 extern void ic_type_convert( Handle self, Byte * dstData, PRGBColor dstPal, int dstType, int * palSize, Bool palSize_only);
@@ -352,6 +357,7 @@ extern Bool img_region_foreach(
 extern Bool img_point_in_region( int x, int y, PRegionRec region);
 extern PRegionRec img_region_polygon( Point *pts, int count, int rule);
 extern PRegionRec img_region_mask( Handle mask);
+extern void img_region_offset( PRegionRec region, int dx, int dy);
 
 /* istXXX function */
 typedef double FilterFunc( const double x );
