@@ -23,20 +23,6 @@ tesselation in xrender is horribly broken, but for now I'll give it a go to at l
 source is from https://github.com/freedesktop/xorg-libXrender/blob/master/src/Poly.c, (c) Keith Packard
 
 */
-int
-my_XRenderCompositeDoublePoly (Display		    *dpy,
-			    int			    op,
-			    Picture		    src,
-			    Picture		    dst,
-			    _Xconst XRenderPictFormat	*maskFormat,
-			    int			    xSrc,
-			    int			    ySrc,
-			    int			    xDst,
-			    int			    yDst,
-			    _Xconst XPointDouble    *fpoints,
-			    int			    npoints,
-			    int			    winding);
-
 typedef struct {
 	Picture   picture;
 	Pixmap    pixmap;
@@ -874,7 +860,7 @@ my_XRenderCompositeDoublePoly (Display		    *dpy,
 
 #else
 
-Bool prima_init_xrender_subsystem(char * error_buf)                 { return true; }
+Bool prima_init_xrender_subsystem(char * error_buf, Bool disable_argb32) { return true; }
 void prima_done_xrender_subsystem(void)                             { }
 Bool apc_gp_aa_fill_poly( Handle self, int numPts, NPoint * points) { return false; }
 Bool apc_gp_aa_bars( Handle self, int nr, NRect *rr)                { return false; }
