@@ -1,264 +1,296 @@
-#ifdef __CYGWIN__
-
-#define SOCKET int
-#define _get_osfhandle(a) a
-
-#include <pthread.h>
-
-#else
-
-#include <winsock.h>
-
-void my_fd_zero( fd_set* f)           { FD_ZERO( f); }
+#include <winsock2.h>
+#include <ws2tcpip.h>
 
+#ifndef SO_OPENTYPE
+#define SO_OPENTYPE 0x7008
 #endif
 
-typedef fd_set type_fd_set;
-void std_fd_set( int fd, fd_set * f) { FD_SET(fd, f); }
-
 #include "win32\win32guts.h"
 #ifndef _APRICOT_H_
 #include "apricot.h"
 #endif
 #include "guts.h"
-#include "Component.h"
 #include "File.h"
 
-void my_fd_set( HANDLE fd, type_fd_set * f) { std_fd_set( PTR2UV(fd), f); }
-
 #define var (( PFile) self)->
-#define  sys (( PDrawableData)(( PComponent) self)-> sysData)->
-#define  dsys( view) (( PDrawableData)(( PComponent) view)-> sysData)->
+#define  sys (( PDrawableData)(( PFile) self)-> sysData)->
+#define  dsys( view) (( PDrawableData)(( PFile) view)-> sysData)->
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#ifndef __CYGWIN__
+#undef  getsockopt
 
-#undef  select
-#undef  fd_set
-#undef  FD_ZERO
-#define FD_ZERO my_fd_zero
-#undef  FD_SET
-#define FD_SET my_fd_set
-
-#endif
-
-static Bool            socket_thread_started = false;
-static Bool            socket_set_changed    = false;
-static struct timeval  socket_timeout        = {0, 200000};
-static char            socket_err_buf[256];
-
-static fd_set socket_set1[3];
-static fd_set socket_set2[3];
-static int    socket_commands[3] = { feRead, feWrite, feException};
-
-void
-#ifdef __CYGWIN__
-*
-#endif
-socket_select( void *dummy)
-{
-	int count;
-	while ( !prima_guts.app_is_dead) {
-		if ( socket_set_changed) {
-			// updating  handles
-			int i;
-			if ( WaitForSingleObject( guts. socket_mutex, INFINITE) != WAIT_OBJECT_0) {
-				strcpy( socket_err_buf, "Failed to obtain socket mutex ownership for thread #2");
-				PostThreadMessage( guts. main_thread_id, WM_CROAK, 1, ( LPARAM) &socket_err_buf);
-				break;
-			}
-			for ( i = 0; i < 3; i++)
-				memcpy( socket_set1+i, socket_set2+i, sizeof( fd_set));
-			socket_set_changed = false;
-			ReleaseMutex( guts. socket_mutex);
-		}
-
-		// calling select()
-#ifndef __CYGWIN__
-		count = socket_set1[0]. fd_count + socket_set1[1]. fd_count + socket_set1[2]. fd_count;
-#else
-		count = 0;
-		{
-			int i,j;
-			for ( i = 0; i < FD_SETSIZE; i++)
-				for ( j = 0; j < 3; i++)
-					if ( FD_ISSET( i, socket_set1+j)) {
-						count++;
-						goto END;
-					}
-END:;
-		}
-#endif
-		if ( count > 0) {
-			int i, j, result = select( FD_SETSIZE-1, &socket_set1[0], &socket_set1[1], &socket_set1[2], &socket_timeout);
-			socket_set_changed = true;
-			if ( result == 0) continue;
-			if ( result < 0) {
-				int err;
-#ifndef __CYGWIN__
-				if (( err = WSAGetLastError()) == WSAENOTSOCK)
-#else
-				if (( err = errno) == EBADF)
-#endif
-				{
-					// possibly some socket was closed
-					guts. socket_post_sync = 1;
-					PostThreadMessage( guts. main_thread_id, WM_SOCKET_REHASH, 0, 0);
-					while( guts. socket_post_sync) Sleep(1);
-				} else {
-					// some error
-					char * msg;
-#ifndef __CYGWIN__
-					msg = err_msg( err, socket_err_buf);
-#else
-					strlcpy( msg = socket_err_buf, strerror(err), 255);
-#endif
-					PostThreadMessage( guts. main_thread_id, WM_CROAK, 0, (LPARAM) msg);
-				}
-				continue;
-			}
-			// posting select() results
-			for ( j = 0; j < 3; j++)
-#ifndef __CYGWIN__
-				for ( i = 0; i < socket_set1[j]. fd_count; i++) {
-#else
-				for ( i = 0; i < FD_SETSIZE; i++) {
-					if ( !FD_ISSET( i, socket_set1 + j)) continue;
-#endif
-					guts. socket_post_sync = 1;
-					PostThreadMessage( guts. main_thread_id, WM_SOCKET, socket_commands[j],
-#ifndef __CYGWIN__
-						( LPARAM) socket_set1[j]. fd_array[i]
-#else
-						( LPARAM) i
-#endif
-					);
-					while( guts. socket_post_sync) Sleep(1);
-				}
-		} else
-			// nothing to 'select', sleeping
-			Sleep( socket_timeout. tv_sec * 1000 + socket_timeout. tv_usec / 1000);
-	}
-
-	// if somehow failed, making restart possible
-	socket_thread_started = false;
-#ifdef __CYGWIN__
-	return NULL;
-#endif
-}
+static int  socket_version = 0;
+static List  ts;
+static List  th;
+static List  tf;
+static PHash socket_events = NULL;
 
+typedef struct {
+	WINHANDLE event;
+	int       refcnt;
+	int       effective_mask;
+} SocketEventRec, *PSocketEventRec;
 
 static void
-reset_sockets( void)
+reset_syshandles(void)
 {
 	int i;
 
-	// enter section
-	if ( socket_thread_started) {
-		if ( WaitForSingleObject( guts. socket_mutex, INFINITE) != WAIT_OBJECT_0)
-			croak("Failed to obtain socket mutex ownership for thread #1");
-	}
-
-	// copying handles
-	for ( i = 0; i < 3; i++)
-		FD_ZERO( &socket_set2[i]);
-
-	for ( i = 0; i < guts. sockets. count; i++) {
-		Handle self = guts. sockets. items[i];
-		if ( var eventMask & feRead)
-			FD_SET( sys s. file. object, &socket_set2[0]);
-		if ( var eventMask & feWrite)
-			FD_SET( sys s. file. object, &socket_set2[1]);
-		if ( var eventMask & feException)
-			FD_SET( sys s. file. object, &socket_set2[2]);
-	}
-
-	socket_set_changed = true;
-
-	// leave section and start the thread, if needed
-	if ( !socket_thread_started) {
-		if ( !( guts. socket_mutex = CreateMutex( NULL, FALSE, NULL))) {
-			apiErr;
-			croak("Failed to create socket mutex object");
+	for ( i = 0; i < th.count; i++) {
+		Handle self = th.items[i];
+		switch ( sys s.file.type ) {
+		case FHT_STDIN:
+			if ( var eventMask & feRead) {
+				/* apc_file_attach is responsible for select_handles[] not overflowing */
+				select_handles[ select_n_handles++ ] = (HANDLE) sys s.file.object;
+				return;
+			}
 		}
-#ifndef __CYGWIN__
-		guts. socket_thread = ( HANDLE) _beginthread( socket_select, 40960, NULL);
+	}
+}
+
+static Bool
+fill_socket_vector( void * item, int keyLen, void * key, void * params)
+{
+	PSocketEventRec ev = (PSocketEventRec ) item;
+	/* apc_file_attach is responsible for select_handles[] not overflowing */
+	if ( ev-> effective_mask != 0 ) {
+		select_handles[ select_n_handles++ ] = ev-> event;
+		if ( debug ) {
+			char buf[64];
+			snprintf( buf, 64, "%p ", (void*) ev->event);
+			strcat(( char*) params, buf);
+		}
+	}
+	return false;
+}
+
+static void
+reset_all_handles(void)
+{
+	char debug_buf[MAX_SELECT_HANDLES * ( 4 + sizeof(void*) * 2 )] = "";
+	select_n_handles = 0;
+	reset_syshandles();
+	hash_first_that( socket_events, fill_socket_vector, debug_buf, NULL, NULL );
+	if ( debug ) warn("reset socket events: %s\n", debug_buf);
+}
+
+static void
+dispatch_file_event( Handle self, int cmd)
+{
+	Event ev;
+	if (debug) warn("main: dealing cmd=%s to %s.%p" ,
+		( cmd == cmFileRead) ? "cmFileRead" :
+			(( cmd == cmFileWrite) ? "cmFileWrite" : "cmFileException"),
+		var name,
+		(void*)self
+	);
+	ev. cmd = cmd;
+	if ( var stage == csNormal )
+		CComponent(self)-> message( self, &ev);
+}
+
+static void
+dispatch_file_msg( PList t, int mask, WINHANDLE src)
+{
+	int i;
+	for ( i = 0; i < t->count; i++) {
+		Handle self = t->items[ i];
+		if (
+			( sys s.file.event == src) &&
+			( var eventMask & mask )
+		) {
+			dispatch_file_event(self,
+				( mask == feRead)   ? cmFileRead :
+				(( mask == feWrite) ? cmFileWrite : cmFileException)
+			);
+			break;
+		}
+	}
+}
+
+Bool
+file_process_events(int cmd, WPARAM param1, LPARAM param2)
+{
+	Bool events_handled = false;
+
+	switch (cmd) {
+	case WM_FILE_REHASH:
+		{
+			int i;
+			for ( i = 0; i < tf.count; i++) {
+				Handle self = tf.items[i];
+				CFile( self)-> is_active( self, true);
+			}
+			for ( i = 0; i < tf.count; i++) {
+				Handle self = tf.items[i];
+				if ( var eventMask & feRead)
+					PostMessage( NULL, WM_FILE, feRead, ( LPARAM) self);
+				if ( var eventMask & feWrite)
+					PostMessage( NULL, WM_FILE, feWrite, ( LPARAM) self);
+			}
+			PostMessage( NULL, WM_FILE_REHASH, 0, 0);
+		}
+		break;
+	case WM_FILE:
+		dispatch_file_msg(&tf, param1, (WINHANDLE) param2);
+		break;
+	}
+
+	return events_handled;
+}
+
+Bool
+process_file_msg( WINHANDLE src)
+{
+	int i, available_events = 0;
+	WSANETWORKEVENTS ev;
+	dispatch_file_msg( &th, feRead, src );
+
+	for ( i = 0; i < ts.count; i++) {
+		Handle self = ts.items[ i];
+		if ( sys s.file.event != src)
+			continue;
+		if ( debug ) warn(
+			"message to file(%p) in socket(%p), event(%p)\n", 
+			(void*)self, (void*)sys s.file.object, (void*)sys s.file.event
+		);
+
+		if ( WSAEnumNetworkEvents((SOCKET) sys s.file.object, (WSAEVENT) sys s.file.event, &ev) != 0 ) {
+			rc = WSAGetLastError();
+			apcWarn;
+			return false;
+		}
+
+		/* WSAEnumNetworkEvents returns usable value only once */
+		available_events = 0;
+		if (ev.lNetworkEvents & (FD_ACCEPT | FD_READ | FD_CLOSE))
+			available_events |= feRead;
+		if (ev.lNetworkEvents & (FD_CONNECT | FD_WRITE))
+			available_events |= feWrite;
+		if (ev.lNetworkEvents & FD_OOB)
+			available_events |= feException;
+		if ( debug ) warn( "available events: %x\n", available_events);
+		break;
+	}
+
+	if ( available_events == 0 ) return true;
+
+	for ( i = 0; i < ts.count; i++) {
+		Handle self = ts.items[ i];
+		if ( sys s.file.event != src)
+			continue;
+
+		protect_object(self);
+		if ( available_events & feRead)
+			dispatch_file_event( self, cmFileRead );
+		if ( available_events & feWrite)
+			dispatch_file_event( self, cmFileWrite );
+		if ( available_events & feException)
+			dispatch_file_event( self, cmFileException );
+		unprotect_object(self);
+	}
+
+	return true;
+}
+
+static int
+mask2wsa(int src)
+{
+	long dst = 0;
+	if ( src & feRead )
+		dst |= FD_ACCEPT | FD_READ | FD_CLOSE;
+	if ( src & feWrite )
+		dst |= FD_CONNECT | FD_WRITE;
+	if ( src & feException )
+		dst |= FD_OOB;
+	return dst;
+}
+
+
+/* more than one socket must refer to same event, with a combined mask,
+because WSAEventSelect(s,e1) then WSAEventSelect(s,e2) doesn't work */
+static void
+socket_event_update_mask( Handle self)
+{
+	int i, mask;
+	PSocketEventRec ev;
+	if ( ( ev = (PSocketEventRec) hash_fetch( socket_events, &sys s.file.object, sizeof( sys s.file.object))) == NULL)
+		return;
+
+	for ( i = 0, mask = 0; i < ts.count; i++) {
+		PFile f = (PFile) ts.items[i];
+		if ( dsys(f)s.file.object == sys s.file.object)
+			mask |= f->eventMask;
+	}
+
+	if ( mask != ev-> effective_mask ) {
+		if ( debug ) warn(
+			"socket(%p),event(%p) changed mask from %x to %x\n",
+			(void*)sys s.file.object, (void*)ev->event, ev->effective_mask, mask
+		);
+		ev-> effective_mask = mask;
+		if ( WSAEventSelect((SOCKET ) sys s.file.object, (WSAEVENT) ev->event, mask2wsa(ev-> effective_mask)) != 0 ) {
+			rc = WSAGetLastError();
+			apcWarn;
+		}
+	}
+}
+
+Bool
+file_subsystem_init( void)
+{
+	int  _data, _sz = sizeof(int);
+	(void)_data;
+	(void)_sz;
+#ifdef PERL_OBJECT     /* init perl socket library, if any */
+	PL_piSock-> Htons( 80);
 #else
-		pthread_create(( pthread_t*) &guts. socket_thread, 0, socket_select, NULL);
+	win32_htons(80);
 #endif
-		socket_thread_started = true;
-	} else
-		ReleaseMutex( guts. socket_mutex);
+	if ( getsockopt(( SOCKET) INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE, (char*)&_data, &_sz) != 0)
+		socket_version = -1; /* no sockets available */
+	else
+		socket_version = 1;
+
+	list_create(&th, 8, 8);
+	list_create(&ts, 8, 8);
+	list_create(&tf, 8, 8);
+	socket_events = prima_hash_create();
+
+	return true;
 }
 
 void
-socket_rehash( void)
+file_subsystem_done( void)
 {
-	int i;
-	for ( i = 0; i < guts. sockets. count; i++) {
-		Handle self = guts. sockets. items[i];
-		CFile( self)-> is_active( self, true);
-	}
+	prima_hash_destroy( socket_events, true );
+	list_destroy(&tf);
+	list_destroy(&ts);
+	list_destroy(&th);
 }
 
-
 Bool
 apc_file_attach( Handle self)
 {
 	int fhtype;
+	PSocketEventRec ev;
 	objCheck false;
 
-	if ( PFile(self)->fd > FD_SETSIZE ) return false;
-
-	if ( guts. socket_version == 0) {
-		int  _data, _sz = sizeof( int);
-		(void)_data;
-		(void)_sz;
-#ifdef __CYGWIN__
-		_sz = htons(80);
-		guts. socket_version = 2;
-#else
-#ifdef PERL_OBJECT     // init perl socket library, if any
-		PL_piSock-> Htons( 80);
-#else
-		win32_htons(80);
-#endif
-		if ( getsockopt(( SOCKET) INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE, (char*)&_data, &_sz) != 0)
-			guts. socket_version = -1; // no sockets available
-		else
-#if PERL_PATCHLEVEL < 8
-			guts. socket_version = ( _data == SO_SYNCHRONOUS_NONALERT) ? 1 : 2;
-#else
-			guts. socket_version = 1;
-#endif
-
-#endif
-	}
-
-	if ( SOCKETS_NONE)
+	if (socket_version == -1)
 		return false;
 
-	sys s. file. object = SOCKETS_AS_HANDLES ?
-		(( SOCKETHANDLE) _get_osfhandle( var fd)) :
-		((INT2PTR(SOCKETHANDLE, var fd)));
+	sys s.file.object = _get_osfhandle( var fd);
 
-	{
-		int  _data, _sz = sizeof( int);
-		int result =
-#ifndef __CYGWIN__
-			SOCKETS_AS_HANDLES ?
-			WSAAsyncSelect((SOCKET) sys s. file. object, (HWND) NULL, 0, 0) :
-#endif
-			getsockopt(( SOCKET) sys s. file. object, SOL_SOCKET, SO_TYPE, (char*)&_data, &_sz);
+	if ( GetStdHandle( STD_INPUT_HANDLE ) == (HANDLE) sys s.file.object ) {
+		fhtype = FHT_STDIN;
+	} else {
+		int result = WSAAsyncSelect((SOCKET) sys s. file. object, (HWND) NULL, 0, 0);
 		if ( result != 0)
-#ifndef __CYGWIN__
 			fhtype = ( WSAGetLastError() == WSAENOTSOCK) ? FHT_OTHER : FHT_SOCKET;
-#else
-			fhtype = ( errno == EBADF) ? FHT_OTHER : FHT_SOCKET;
-#endif
 		else
 			fhtype = FHT_SOCKET;
 	}
@@ -267,13 +299,36 @@ apc_file_attach( Handle self)
 
 	switch ( fhtype) {
 	case FHT_SOCKET:
-		list_add( &guts. sockets, self);
-		reset_sockets();
+		if ( select_n_handles >= MAX_SELECT_HANDLES )
+			return false;
+		list_add( &ts, self);
+		if ( !( ev = (PSocketEventRec) hash_fetch( socket_events, &sys s.file.object, sizeof( sys s.file.object)))) {
+			ev = malloc( sizeof(SocketEventRec));
+			memset( ev, 0, sizeof(SocketEventRec));
+			ev-> event = (WINHANDLE) WSACreateEvent();
+			hash_store( socket_events, &sys s.file.object, sizeof( sys s.file.object ), ev);
+		}
+
+		if ( debug ) warn(
+			"file(%p) is socket(%p) and event(%p)\n", 
+			(void*)self, (void*)sys s.file.object, (void*)ev->event
+		);
+		sys s.file.event = ev-> event;
+		ev-> refcnt++;
+		socket_event_update_mask(self);
+		reset_all_handles();
+		break;
+	case FHT_STDIN:
+		if ( th.count == 0 && select_n_handles >= MAX_SELECT_HANDLES )
+			return false;
+		sys s.file.event = (WINHANDLE) sys s.file.object;
+		list_add( &th, self);
+		reset_all_handles();
 		break;
 	default:
-		if ( guts. files. count == 0)
-			PostMessage( NULL, WM_FILE, 0, 0);
-		list_add( &guts. files, self);
+		if ( tf.count == 0)
+			PostMessage( NULL, WM_FILE_REHASH, 0, 0);
+		list_add( &tf, self);
 		break;
 	}
 
@@ -283,13 +338,24 @@ apc_file_attach( Handle self)
 Bool
 apc_file_detach( Handle self)
 {
+	PSocketEventRec ev;
 	switch ( sys s. file. type) {
 	case FHT_SOCKET:
-		list_delete( &guts. sockets, self);
-		reset_sockets();
+		if ( ( ev = (PSocketEventRec) hash_fetch( socket_events, &sys s.file.object, sizeof( sys s.file.object)))) {
+			if ( 0 == --ev-> refcnt ) {
+				WSACloseEvent(( WSAEVENT ) ev-> event );
+				hash_delete( socket_events, &sys s.file.object, sizeof( sys s.file.object), true);
+			}
+		}
+		list_delete( &ts, self);
+		reset_all_handles();
+		break;
+	case FHT_STDIN:
+		list_delete( &th, self);
+		reset_all_handles();
 		break;
 	default:
-		list_delete( &guts. files, self);
+		list_delete( &tf, self);
 	}
 	return true;
 }
@@ -299,7 +365,11 @@ apc_file_change_mask( Handle self)
 {
 	switch ( sys s. file. type) {
 	case FHT_SOCKET:
-		reset_sockets();
+		socket_event_update_mask(self);
+		reset_all_handles();
+		break;
+	case FHT_STDIN:
+		reset_all_handles();
 		break;
 	default:;
 	}
@@ -309,43 +379,6 @@ apc_file_change_mask( Handle self)
 PList
 apc_getdir( const char *dirname, Bool is_utf8)
 {
-#ifdef __CYGWIN__
-	DIR *dh;
-	struct dirent *de;
-	PList dirlist = NULL;
-	char *type, *dname;
-	char path[ 2048];
-	struct stat s;
-
-	if ( *dirname == '/' && dirname[1] == '/') dirname++;
-	if ( strcmp( dirname, "/") == 0)
-		dname = "";
-	else
-		dname = ( char*) dirname;
-
-	if (( dh = opendir( dirname)) && (dirlist = plist_create( 50, 50))) {
-		while (( de = readdir( dh))) {
-			list_add( dirlist, (Handle)duplicate_string( de-> d_name));
-			snprintf( path, 2047, "%s/%s", dname, de-> d_name);
-			type = NULL;
-			if ( stat( path, &s) == 0) {
-				switch ( s. st_mode & S_IFMT) {
-				case S_IFIFO:        type = "fifo";  break;
-				case S_IFCHR:        type = "chr";   break;
-				case S_IFDIR:        type = "dir";   break;
-				case S_IFBLK:        type = "blk";   break;
-				case S_IFREG:        type = "reg";   break;
-				case S_IFLNK:        type = "lnk";   break;
-				case S_IFSOCK:       type = "sock";  break;
-				}
-			}
-			if ( !type) type = "reg";
-			list_add( dirlist, (Handle)duplicate_string( type));
-		}
-		closedir( dh);
-	}
-	return dirlist;
-#else
 	long		 len;
 	WCHAR		 scanname[(MAX_PATH+3)*2];
 	WIN32_FIND_DATAW FindData;
@@ -429,7 +462,6 @@ apc_getdir( const char *dirname, Bool is_utf8)
 #undef FILE
 #undef DIR
 	return ret;
-#endif
 }
 
 static WCHAR*
@@ -481,9 +513,11 @@ win32_set_errno(void)
 	case ERROR_DISK_FULL:
 		errno = ENOSPC;
 		break;
+#ifdef EDQUOT
 	case ERROR_NOT_ENOUGH_QUOTA:
 		errno = EDQUOT;
 		break;
+#endif
 	default: /* ERROR_INVALID_FUNCTION - eg. on a FAT volume */
 		errno = EINVAL;
 		break;
